﻿// SummitConfig.js
//  This module manages the different configurations of Summit components
//  As of v0.20.x, this module handles: 
//      - Loading of analog formulas
//      - Serial port ordering
//

// Dependencies and Globals
//#region Dependencies and Globals
var moment = require('moment');
var fs = require('fs');
var cp = require('child_process');

var Logger;// Logging global

var SummitInfo = null;
var SummitConfiguration = null;

// Default version as of 18-04-15
var DefaultSummitConfiguration = {
    "trio": {
        "installed": true,
        "version": "3-1801"
    },
    "analog": {
        "installed": true,
        "version": "3-1801"
    },
    "digital": {
        "installed": true,
        "version": "3-1801"
    },
    "serial": {
        "installed": true,
        "version": "1-1537"
    },
    "summitio": {
        "installed": false,
        "version": ""
    }
}
var DefaultAnalogVoltageFormulae = {
    "name": "Trio + Analog (legacy)",
    "inputs": {
        "slope": 0.0114,
        "offset": -2.2011
    },
    "outputs": {
        "slope": 200.0,
        "offset": 0.0
    },
    "trio": {
        "installed": true,
        "version": "3-1525"
    },
    "analog": {
        "installed": true,
        "version": "3-1540"
    },
    "summitio": {
        "installed": false,
        "version": ""
    }
}

//#endregion Dependencies and Globals

// Initialize
//#region Initialize
exports.Initialize = function (logger, summitInfo, callback) {
    Logger = logger;// Globalize

    ReadSummitInfo(summitInfo, function () {
        ReadSummitConfigurationFile(function () {
            callback(SummitInfo, SummitConfiguration);// Return the global SummitInfo and SummitConfiguration
        });// Read the Summit configuration file
    });// Read the Summit name
}

function ReadSummitConfigurationFile(callback) {

    try {
        // Try to read the Summit configuration file ... set global SummitConfiguration
        SummitConfiguration = require("./factory/summitconfig.json");
        Logger.info("SummitConfig.ReadSummitConfigurationFile: Summit Configuration=" + JSON.stringify(SummitConfiguration));
        callback();
    }
    catch (error) {
        // This should never happen because this file is always loaded at the time of production
        //  In case it does, we'll load a hardcoded value of the configuration ... set global SummitConfiguration
        Logger.error("SummitConfig.ReadSummitConfigurationFile: Unable to find Summit configuration file, error:" + error + ". Loading hard-coded Summit configuration=" + JSON.stringify(DefaultSummitConfiguration));
        SummitConfiguration = DefaultSummitConfiguration;
        callback();
    }
}
//#endregion Initialize

//#region Summit Name
function ReadSummitInfo(summitInfo, callback) {

    try {
        // Try to read the Summit configuration file ... set global SummitInfo
        SummitInfo = require("./factory/summitinfo.json");
        Logger.info("SummitConfig.ReadSummitInfoFile: Summit Name=" + JSON.stringify(SummitInfo));
        callback();
    }
    catch (error) {
        // This will happen on the very first attempt, at which time, we'll load it with the MACID for future use
        Logger.error("SummitConfig.ReadSummitInfoFile: Unable to find Summit name file, error:" + error + ". Initializing Summit name file with MACID");

        SummitInfo = summitInfo;
        fs.writeFile('./factory/summitinfo.json', JSON.stringify(SummitInfo), function (error) {
            if (error) {
                Logger.error("SummitConfig.ReadSummitInfo: Summit name write error. Error=" + error);
            }
            Logger.info("SummitConfig.ReadSummitInfo: Summit name updated=" + JSON.stringify(SummitInfo));
        });
        callback();
    }
}

exports.WriteSummitInfo = function (summitInfo, callback) {
    SummitInfo = summitInfo;
    fs.writeFile('./factory/summitinfo.json', JSON.stringify(SummitInfo), function (error) {
        if (error) {
            Logger.error("SummitConfig.SetSummitInfo: Summit name write error. Error=" + error);
        }
        Logger.info("SummitConfig.SetSummitInfo: Summit name updated=" + JSON.stringify(SummitInfo));
        callback(SummitInfo);
    });
}
//#endregion Summit Name

// Determine the analog voltage formulae to use
//#region Get Analog Voltage Formulae
exports.GetAnalogVoltageFormulae = function (summitConfiguration, callback) {

    try {
        // Try to read the Summit configuration file ...
        var AnalogVoltageFormulaeArray = require("./factory/analogformulae.json");
        Logger.info("SummitConfig.GetAnalogVoltageFormulae: Read list of analog formulae=" + JSON.stringify(AnalogVoltageFormulaeArray));

        //#region Determine analog formulae
        //  This section looks at the Summit configuration and the AnalogFormulaeArray 
        //  to determine which analog voltage formulae to pass back to the core code
        //  The passed back formulae will be used by analogInputs and analogOutputs

        //  Find out which analog board we are using: Analog or Summit
        var AnalogVoltageFormulae = null;
        if (summitConfiguration.analog.installed) {
            //  Rather than sort and do this algorithmically, since at the time of this feature, I know that there is only two analog IOs, 
            //  I'm going to assume that if the installed board is less than 3-1801, it's legacy, else it's the new board.
            //  We'll address more elaborate search methods later .. but maybe never!
            var installedtrioboard = summitConfiguration.trio.version.split('-');
            var installedtrioboardversion = installedtrioboard[0];
            var installedtrioboardbuild = installedtrioboard[1];
            //Logger.error("SummitConfig.GetAnalogVoltageFormulae: Installed Trio board version=" + installedtrioboardversion + " build=" + installedtrioboardbuild + " length=" + AnalogVoltageFormulaeArray.formulae.length);
            for (var i = 0; i < AnalogVoltageFormulaeArray.formulae.length; i++) {
                var formulatrioboard = AnalogVoltageFormulaeArray.formulae[i].trio.version.split('-');
                var formulatrioboardversion = formulatrioboard[0];
                var formulatrioboardbuild = formulatrioboard[1]
                //Logger.error("SummitConfig.GetAnalogVoltageFormulae: Formula name=" + AnalogVoltageFormulaeArray.formulae[i].name + " version=" + formulatrioboardversion + " build=" + formulatrioboardbuild);
                if (AnalogVoltageFormulaeArray.formulae[i].analog.installed) {
                    // Case 1: Legacy Trio & Analog board
                    if (parseInt(installedtrioboardbuild) < 1801 && parseInt(formulatrioboardbuild) < 1801) {
                        AnalogVoltageFormulae = AnalogVoltageFormulaeArray.formulae[i];
                        //Logger.info("SummitConfig.GetAnalogVoltageFormulae: Loading (" + AnalogVoltageFormulae.name + ")=" + JSON.stringify(AnalogVoltageFormulae));
                        break;
                    }
                    // Case 2: Newer Trio and Analog board
                    else if (parseInt(installedtrioboardbuild) >= 1801 && parseInt(formulatrioboardbuild) >= 1801) {
                        AnalogVoltageFormulae = AnalogVoltageFormulaeArray.formulae[i];
                        //Logger.info("SummitConfig.GetAnalogVoltageFormulae: Loading (" + AnalogVoltageFormulae.name + ")=" + JSON.stringify(AnalogVoltageFormulae));
                        break;
                    }
                }
            }
        }
        else if (summitConfiguration.summitio.installed) {
            L//ogger.error("SummitConfig.GetAnalogVoltageFormulae: Summit IO board installed");
            // We are using Summit IO
            //  Rather than sort and do this algorithmically, since at the time of this feature, I know that there is only one summit, 
            //  I'm going to assume that there is only one formula to select from. We'll address this later ... but maybe never!
            for (var i = 0; i < AnalogVoltageFormulaeArray.formulae.length; i++) {
                if (AnalogVoltageFormulaeArray.formulae[i].summitio.installed) {
                    AnalogVoltageFormulae = AnalogVoltageFormulaeArray.formulae[i];
                    //Logger.info("SummitConfig.GetAnalogVoltageFormulae: Loading (" + AnalogVoltageFormulae.name + ")=" + JSON.stringify(AnalogVoltageFormulae));
                    break;
                }
            }
        }
        else {
            Logger.error("SummitConfig.GetAnalogVoltageFormulae: Unknown board installed according Summit Configuration, loading hard-coded formulae.");
            // This should never happen, so just hard-code it!
            AnalogVoltageFormulae = DefaultAnalogVoltageFormulae;
        }
        //#endregion Determine analog formulae

        callback(AnalogVoltageFormulae);
    }
    catch (error) {
        // This should never happen because this file should always be in the /factory directory
        //  In case of failure, we'll load the default value hard-coded in this module
        Logger.error("SummitConfig.GetAnalogVoltageFormulae: Error loading analog formulae, loading hard-coded formulae. " + error);
        AnalogVoltageFormulae = DefaultAnalogVoltageFormulae;
        callback(AnalogVoltageFormulae);
    }
}
//#endregion Get Analog Voltage Formulae